//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Reflection;
using System.Collections;
using System.Security.Permissions;
using System.Runtime.Serialization;

namespace AssemblySample
{

	class AssemblySampleMain
	{
		static void Main(string[] args)
		{
			try
			{
				// Assembly a = Assembly.Load("SubAssembly, SN=d8d19842315c20be");
				// Assembly a = Assembly.LoadFrom(@"..\..\SubAssembly\bin\debug\SubAssembly.dll");
				// Assembly a = Assembly.LoadFrom(@"..\..\bin\debug\SubAssembly.dll");
				Console.WriteLine(Assembly.CreateQualifiedName("SubAssembly", "MyClass"));
				Assembly a = Assembly.Load("SubAssembly");
				Console.WriteLine("Listing all of the types for");
				Console.WriteLine(" {0}", a.FullName);
				Type [] types = a.GetTypes();
				foreach(Type t in types)
				{
					Console.WriteLine("Type: {0}", t);
				}
				Module [] modules = a.GetLoadedModules();
				foreach(Module m in modules)
				{
					Console.WriteLine("Loaded Modules: {0}", m);
				}

				Console.WriteLine("CodeBase: {0}", a.CodeBase);
				Console.WriteLine("Location: {0}", a.Location);
				Console.WriteLine("EntryPoint: {0}", a.EntryPoint);
				Console.WriteLine("Evidence count: {0}", a.Evidence.Count);
				IEnumerator ee = a.Evidence.GetEnumerator();
				while(ee.MoveNext())
				{
					Console.WriteLine(ee.Current);
				}
				Console.WriteLine("File loaded from GAC: {0}", a.GlobalAssemblyCache);
				Object [] attributes = a.GetCustomAttributes(true);
				foreach(object o in attributes)
				{
					Console.WriteLine(o);
				}
				TypeSampler.MyClass mc = new TypeSampler.MyClass();
				Console.WriteLine("MyInt is: {0}", mc.MyInt);

				AssemblyName [] assemblyNames = a.GetReferencedAssemblies();
				foreach(AssemblyName an in assemblyNames)
				{
					Console.WriteLine("Referenes: {0}", an.Name);
				}
				a = Assembly.GetAssembly(typeof(double));
				Console.WriteLine(a.FullName);
				a = Assembly.GetAssembly(typeof(TypeSampler.MyClass));
				Console.WriteLine(a.FullName);
			}
			catch(Exception e)
			{
				Console.WriteLine(e.ToString());
			}
		}
	}
}
