//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.IO;
using System.CodeDom;
using System.CodeDom.Compiler;

class GenerateCode
{
	public static void HelloWorldCode(CodeDomProvider provider, StreamWriter output)
	{
		// Generate code
		ICodeGenerator cg = provider.CreateGenerator(output);
		CodeGeneratorOptions op = new CodeGeneratorOptions();

		// Generate the comments at the beginning of the function
		CodeCommentStatement comment = new CodeCommentStatement("Code to generate Hello World!");
		cg.GenerateCodeFromStatement(comment, output, op);
		comment = new CodeCommentStatement("First generate the namespace.");
		cg.GenerateCodeFromStatement(comment, output, op);

		// The namespace
		CodeNamespace codeNamespace = new CodeNamespace("HelloWorld");
		// The namespace
		CodeNamespaceImport import = new CodeNamespaceImport("System");
		codeNamespace.Imports.Add(import);
		// The class is named with a unique name
		CodeTypeDeclaration mainClass = new CodeTypeDeclaration("HelloWorldMain");

		// Add the class to the namespace
		codeNamespace.Types.Add(mainClass);

		// Set up the Main function
		CodeEntryPointMethod main = new CodeEntryPointMethod();

		CodeParameterDeclarationExpression param = 
			new CodeParameterDeclarationExpression("System.String []", "args");

		comment = new CodeCommentStatement("Output the greeting on the Console");
		main.Statements.Add(comment);

		CodeVariableDeclarationStatement variable = new CodeVariableDeclarationStatement(typeof(string), "message", new CodePrimitiveExpression("Hello World!"));
		main.Statements.Add(variable);

		CodeVariableReferenceExpression[] arg = new CodeVariableReferenceExpression[1];
		arg[0] = new CodeVariableReferenceExpression(variable.Name);
		CodeMethodReferenceExpression type = new CodeMethodReferenceExpression();
		type.MethodName = "Console";
		CodeMethodInvokeExpression methodCall = new CodeMethodInvokeExpression(type,"WriteLine",arg);
		main.Statements.Add(methodCall);

		mainClass.Members.Add(main);

		cg.GenerateCodeFromNamespace(codeNamespace, output, op);

	}
	public static void CompileAndExecute(CodeDomProvider provider, string filename, string target)
	{
		// Now use the CodeProvider interface
		CompilerParameters param = new CompilerParameters(null, target, true);
		param.GenerateExecutable = true;
		ICodeCompiler cc = provider.CreateCompiler();
		CompilerResults cr = cc.CompileAssemblyFromFile(param, filename);
		System.Collections.Specialized.StringCollection output = cr.Output;
		foreach(string s in output)
		{
			Console.WriteLine(s);
		}

		if(cr.Errors.Count != 0)
		{
			CompilerErrorCollection es = cr.Errors;
			foreach(System.CodeDom.Compiler.CompilerError e in es)
			{
				Console.WriteLine(e.ToString());
			}
		}
		else
		{
			// Set ApplicationBase to the current directory
			AppDomainSetup info = new AppDomainSetup();
			info.ApplicationBase = "file:\\\\" + System.Environment.CurrentDirectory;
			// Create an application domain with null evidence
			AppDomain dom = AppDomain.CreateDomain("HelloWorld", null, info);
			dom.ExecuteAssembly(target);
			// Clean up by unloading the application domain
			AppDomain.Unload(dom);  
		}
	}
}
