//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace MatrixMultiplication
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class MatrixMultiplicationMain
	{
		static void Main(string[] args)
		{
			const int niter = 50;
			const int xdim = 100;
			const int ydim = 100;
			double [,] A = new double [,] {{2,3},
										   {3,4},
										   {4,5}};
			double [,] B = new double [,] {{2,3,4},
										   {3,4,5}};
			double [,] C;
			double [,] D;

			MatrixMultiplyDirect direct = new MatrixMultiplyDirect(A, B);
			C = direct.Multiply();
			MatrixMultiplyFixed fix = new MatrixMultiplyFixed(A, B);
			C = D = fix.Multiply();
			MatrixMultiplyLoop loop = new MatrixMultiplyLoop(A, B);
			C = D = loop.Multiply();
			for(int i = 0; i < C.GetLength(0); i++)
			{
				for(int j = 0; j < C.GetLength(1); j++)
				{
					Console.WriteLine("C[{0},{1}] = {2}", i, j, C[i,j]);
				}
			}

			A = new double [ydim, xdim];
			B = new double [ydim, xdim];

			Random r = new Random();

			for(int i = 0; i < ydim; i++)
			{
				for(int j = 0; j < xdim; j++)
				{
					A[i,j] = r.NextDouble();
					B[i,j] = r.NextDouble();
				}
			}

			direct = new MatrixMultiplyDirect(A, B);
			int start,stop;

			start = Environment.TickCount;
			for(int i = 0; i < niter; i++)
				C = direct.Multiply();
			stop = Environment.TickCount;
			Console.WriteLine("Direct ticks: {0} {1} multiplications/second", stop - start, ((double)niter/(double)(stop-start)) * 1000);

			fix = new MatrixMultiplyFixed(A, B);
			start = Environment.TickCount;
			for(int i = 0; i < niter * 100000; i++)
				C = fix.Multiply();
			stop = Environment.TickCount;
			Console.WriteLine("Fixed ticks: {0} {1} multiplications/second", stop - start, ((double)(niter*100000)/(double)(stop-start)) * 1000);

			loop = new MatrixMultiplyLoop(A, B);
			start = Environment.TickCount;
			for(int i = 0; i < niter; i++)
				D = loop.Multiply();
			stop = Environment.TickCount;
			Console.WriteLine("Loop ticks: {0} {1} multiplications/second", stop - start, ((double)(niter)/(double)(stop-start)) * 1000);
			double maxdeviation = 0.0;
			double temp;
			for(int i = 0; i < ydim; i++)
			{
				for(int j = 0; j < xdim; j++)
				{
					temp = C[i,j] - D[i,j];
					if(temp > maxdeviation)
						maxdeviation = temp;
				}
			}
			Console.WriteLine("Maximum deviation: {0}", maxdeviation);
		}
	}
}
