//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace MatrixMultiplication
{
	/// <summary>
	/// Summary description for Matrix.
	/// </summary>
	public abstract class MatrixMultiply
	{
		protected double [,] A;
		protected double [,] B;
		// The result
		protected double [,] C;
		public MatrixMultiply()
		{
		}
		public MatrixMultiply(double [,] A, double [,] B)
		{
			if(A.GetLength(0) != B.GetLength(1) ||
			   A.GetLength(1) != B.GetLength(0))
				throw new ArgumentOutOfRangeException("The two matrixes have incompatible dimensions");

			this.A = new double[A.GetLength(0), A.GetLength(1)];
			this.B = new double[B.GetLength(0), B.GetLength(1)];
			for(int i = 0; i < A.GetLength(0); i++)
				for(int j = 0; j < A.GetLength(1); j++)
					this.A[i,j] = A[i,j];

			for(int i = 0; i < B.GetLength(0); i++)
				for(int j = 0; j < B.GetLength(1); j++)
					this.B[i,j] = B[i,j];
			this.C = new double[A.GetLength(0), B.GetLength(1)];
		}
		public abstract double [,] Multiply();
		public double [,] Multiply(double [,] A, double [,] B)
		{
			if(A.GetLength(0) != B.GetLength(1) ||
				A.GetLength(1) != B.GetLength(0))
				throw new ArgumentOutOfRangeException("The two matrixes have incompatible dimensions");

			this.A = new double[A.GetLength(0), A.GetLength(1)];
			this.B = new double[B.GetLength(0), B.GetLength(1)];
			for(int i = 0; i < A.GetLength(0); i++)
				for(int j = 0; j < A.GetLength(1); j++)
					this.A[i,j] = A[i,j];

			for(int i = 0; i < B.GetLength(0); i++)
				for(int j = 0; j < B.GetLength(1); j++)
					this.B[i,j] = B[i,j];
			this.C = new double[A.GetLength(0), B.GetLength(1)];
			return Multiply();
		}
	}
}
