//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace CustomAttributes
{
	[AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
	class HelloAttribute : Attribute
	{
		private int id;
		private string message;
		public HelloAttribute(string message)
		{
			id = 0;
			this.message = message;
			Console.WriteLine("Constructing Hello");
		}
		public int Id
		{
			get
			{
				return id;
			}
			set
			{
				Console.WriteLine("Setting ID");
				id = value;
			}
		}
		public string Message
		{
			get
			{
				return message;
			}
		}
	}

	[Hello("Hello World!")]
	[Hello("Hello World, again!")]
	class MyClass
	{
		public MyClass()
		{
			Console.WriteLine("Constructing MyClass");
			Object [] attributes = typeof(MyClass).GetCustomAttributes(true);
			foreach(Attribute attrib in attributes)
			{
				HelloAttribute ha = attrib as HelloAttribute;
				if(ha != null)
				{
					Console.WriteLine("Message: {0}", ha.Message);
				}
			}
		}
	}

	class CustomAttributesMain
	{
		static void Main(string[] args)
		{
			MyClass m = new MyClass();
		}
	}
}
