//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Reflection;
using System.Xml.Serialization;

namespace ListTypes
{
	// User defined value type.
	public struct MyStruct
	{
		public int a;
		public int b;
	}

	public enum Sample { Sample1, Sample2, Sample3};

	// Interfaces
	public interface IMyInterface
	{
		// Method returns an array
		string [] MyInterfaceMethod();
	}
	public interface IYourInterface
	{
		void YourInterfaceMethod(MyDelegate callback);
	}

	// Delegate
	public delegate void MyDelegate(string msg);

	// Class
	public class MyClass: IMyInterface, IYourInterface
	{
		// Fields
		private int myInt;
		private string myString;
		private Sample mySample;

		// Constructor
		public MyClass()
		{
			myInt = 0;
			myString = Assembly.GetCallingAssembly().FullName;
		}
		// Method
		public MyStruct MyMethod(int a, int b)
		{
			MyStruct ms = new MyStruct();
			ms.a = a;
			ms.b = b;
			return ms;
		}

		// Interface implementations
		public string [] MyInterfaceMethod()
		{
			string [] returnArray = new string[] {"This", "is", "returned", "from", "MyMethod", "in", "IMyInterface"};
			return returnArray;
		}

		public void YourInterfaceMethod(MyDelegate yourCallback)
		{
			yourCallback("Hello, You!");
		}

		// Property
		public int MyInt
		{
			get
			{
				return myInt;
			}
			set
			{
				myInt = value;
			}
		}

		// Another property
		public string MyString
		{
			get
			{
				return myString;
			}
		}

		// Yet another property
		public Sample MyEnum
		{
			get
			{
				return mySample;
			}
			set
			{
				mySample = value;
			}
		}
	}
	class ListTypesMain
	{
		static void Main(string[] args)
		{
			// Load an assembly using its file name.
			Assembly a = Assembly.GetExecutingAssembly();
			// Get the type names from the assembly.
			Type [] types = a.GetTypes ();
			foreach (Type t in types)
			{
				MemberInfo[] mmi = t.GetMembers(); 
				// Get and display the DeclaringType method. 
				Console.WriteLine("\nThere are {0} members in {1}.", 
					mmi.Length, t.FullName);
				Console.WriteLine("{0} is {1}", t.FullName,
						(t.IsAbstract ? "abstract " : "") + 
						(t.IsArray ? "array " : "") + 
						(t.IsClass ? "class " : "") + 
						(t.IsContextful ? "context " : "") +
						(t.IsInterface ? "interface " : "") +
						(t.IsPointer ? "pointer " : "") +
						(t.IsPrimitive ? "primitive " : "") +
						(t.IsPublic ? "public " : "") +
						(t.IsSealed ? "sealed " : "") +
						(t.IsSerializable ? "serializable " : "") +
						(t.IsValueType ? "value " : ""));
//				Console.WriteLine ("// Members");
//				PrintMembers (mmi);
//				// Static Fields are listed first.
				FieldInfo [] fi;
//				fi = t.GetFields (BindingFlags.Static | 
//					              BindingFlags.NonPublic |
//					              BindingFlags.Public);
//				Console.WriteLine ("// Static Fields");
//				PrintMembers (fi);
//
//				PropertyInfo [] pi;
//				// Static Properties 
//				pi = t.GetProperties (BindingFlags.Static | 
//					                  BindingFlags.NonPublic |
//					                  BindingFlags.Public);
//				Console.WriteLine ("// Static Properties");
//				PrintMembers (pi);
//
//				// Static Events 
//				EventInfo [] ei = t.GetEvents (BindingFlags.Static | 
//					                           BindingFlags.NonPublic |
//					                           BindingFlags.Public);
//				Console.WriteLine ("// Static Events");
//				PrintMembers (ei);
//
//				// Static Methods 
//				MethodInfo [] mi = t.GetMethods (BindingFlags.Static | 
//					                             BindingFlags.NonPublic |
//					                             BindingFlags.Public);
//				Console.WriteLine ("// Static Methods");
//				PrintMembers (mi);
//
//				// Constructors 
//				ConstructorInfo [] ci = t.GetConstructors (BindingFlags.Instance | 
//					                                       BindingFlags.NonPublic |
//					                                       BindingFlags.Public);
//				Console.WriteLine ("// Constructors");
//				PrintMembers (ci);
//
				// Instance Fields 
				fi = t.GetFields (BindingFlags.Instance |
					              BindingFlags.NonPublic | 
					              BindingFlags.Public);
				Console.WriteLine ("// Instance Fields");
				PrintMembers (fi);
//
//				// Instance Properites 
//				pi = t.GetProperties (BindingFlags.Instance |
//					                  BindingFlags.NonPublic | 
//					                  BindingFlags.Public);
//				Console.WriteLine ("// Instance Properties");
//				PrintMembers (pi);
//
//				// Instance Events 
//				ei = t.GetEvents (BindingFlags.Instance |
//					              BindingFlags.NonPublic | 
//					              BindingFlags.Public);
//				Console.WriteLine ("// Instance Events");
//				PrintMembers (ei);
//
//				// Instance Methods 
//				mi = t.GetMethods (BindingFlags.Instance |
//					               BindingFlags.NonPublic | 
//					               BindingFlags.Public);
//				Console.WriteLine ("// Instance Methods");
//				PrintMembers (mi);
			}

			// Now actually create the instances
			MyStruct [] s = new MyStruct [2];
			s[0].a = 1;
			s[0].b = 2;
			s[1].a = 3;
			s[1].b = 4;
		}
		public static void PrintMembers(MemberInfo [] ms) 
		{
			foreach (MemberInfo m in ms) 
			{
				if(m is ConstructorInfo)
				{
					Console.WriteLine ("{0}{1}{2}", "     ",  m, (((MethodBase)m).IsConstructor ? "constructor" : ""));
					Console.WriteLine ("{0} A member of {1}", "     ", ((ConstructorInfo)m).DeclaringType);
					Console.WriteLine ("{0} Calling Convention {1}", "     ", ((ConstructorInfo)m).CallingConvention);
					ParameterInfo [] pi = ((ConstructorInfo)m).GetParameters();
					Console.WriteLine ("{0} There are {1} parameter(s).", "     ", pi.Length);
					foreach (ParameterInfo p in pi) 
					{
						Console.WriteLine ("{0}  {1} {2}", "     ", p.ParameterType, p.Name);
					}	
				}
				else if(m is MethodInfo)
				{
					Console.WriteLine ("{0}{1}", "     ", m);
					Console.WriteLine ("{0} A member of {1}", "     ", ((MethodInfo)m).DeclaringType);
					Console.WriteLine ("{0} Calling Convention {1}", "     ", ((MethodInfo)m).CallingConvention);
					ParameterInfo [] pi = ((MethodInfo)m).GetParameters();
					Console.WriteLine ("{0} There are {1} parameter(s).", "     ", pi.Length);
					foreach (ParameterInfo p in pi) 
					{
						Console.WriteLine ("{0}  {1} {2}", "     ", p.ParameterType, p.Name);
					}	
					Console.WriteLine ("{0} Returns {1}", "     ", ((MethodInfo)m).ReturnType);
				}
				else if(m is FieldInfo)
				{
					Console.WriteLine ("{0}{1}", "     ", m);
					// Get the MethodAttribute enumerated value
					FieldAttributes fieldAttributes = ((FieldInfo)m).Attributes;
					Console.Write ("{0} ", "     ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.Private)
						Console.Write("Private ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.Public)
						Console.Write("Public ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.Assembly)
						Console.Write("Assembly ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.FamANDAssem)
						Console.Write("FamANDAssem ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.Family)
						Console.Write("Family ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.FamORAssem)
						Console.Write("FamORAssem ");
					if((fieldAttributes & FieldAttributes.FieldAccessMask) == FieldAttributes.FamORAssem)
						Console.Write("FamORAssem ");
					if((fieldAttributes & FieldAttributes.HasDefault) != 0)
						Console.Write("HasDefault ");
					if((fieldAttributes & FieldAttributes.HasFieldMarshal) != 0)
						Console.Write("HasFieldMarshal ");
					if((fieldAttributes & FieldAttributes.HasFieldRVA) != 0)
						Console.Write("HasFieldRVA ");
					if((fieldAttributes & FieldAttributes.InitOnly) != 0)
						Console.Write("InitOnly ");
					if((fieldAttributes & FieldAttributes.Literal) != 0)
						Console.Write("Literal ");
					if((fieldAttributes & FieldAttributes.NotSerialized) != 0)
						Console.Write("NotSerialized ");
					if((fieldAttributes & FieldAttributes.PinvokeImpl) != 0)
						Console.Write("PinvokeImpl ");
					if((fieldAttributes & FieldAttributes.RTSpecialName) != 0)
						Console.Write("RTSpecialName ");
					if((fieldAttributes & FieldAttributes.SpecialName) != 0)
						Console.Write("SpecialName ");
					if((fieldAttributes & FieldAttributes.Static) != 0)
						Console.Write("Static ");
					Console.WriteLine (((FieldInfo)m).FieldType);
					if(Attribute.GetCustomAttribute(m, typeof(Attribute)) != null)
						Console.WriteLine ("{0} {1}", "     ", Attribute.GetCustomAttribute(m, typeof(Attribute)));
					Console.WriteLine ("{0} A member of {1}", "     ", ((FieldInfo)m).DeclaringType);
					Attribute attrib = Attribute.GetCustomAttribute(m, typeof(Attribute));
					XmlAttributeAttribute xmlAttrib = attrib as XmlAttributeAttribute;
					if(xmlAttrib != null)
					{
						Console.WriteLine("Name: {0}", xmlAttrib.AttributeName);
						Console.WriteLine("Date Type: {0}", xmlAttrib.DataType);
						Console.WriteLine("Form: {0}", xmlAttrib.Form);
						Console.WriteLine("Namespace: {0}", xmlAttrib.Namespace);
						Console.WriteLine("Type: {0}", xmlAttrib.Type);
						Console.WriteLine("Type ID: {0}", xmlAttrib.TypeId);
					}
				}
				else if(m is PropertyInfo)
				{
					Console.WriteLine ("{0}{1}", "     ", m);
					Console.WriteLine ("{0} CanRead: {1}", "     ", ((PropertyInfo)m).CanRead);
					Console.WriteLine ("{0} CanWrite: {1}", "     ", ((PropertyInfo)m).CanWrite);
					Console.WriteLine ("{0} IsSpecialName: {1}", "     ", ((PropertyInfo)m).IsSpecialName);
					Console.WriteLine ("{0} Member of: {1}", "     ", ((PropertyInfo)m).DeclaringType);
					Console.WriteLine ("{0} Member type: {1}", "     ", ((PropertyInfo)m).MemberType);
					Console.WriteLine ("{0} Name: {1}", "     ", ((PropertyInfo)m).Name);
					Console.WriteLine ("{0} Property type: {1}", "     ", ((PropertyInfo)m).PropertyType);
					Console.WriteLine ("{0} Reflected type: {1}", "     ", ((PropertyInfo)m).ReflectedType);
					if(((PropertyInfo)m).GetGetMethod() != null)
					{
						Console.WriteLine ("{0} Get: {1}", "     ", ((PropertyInfo)m).GetGetMethod());
						Console.WriteLine ("{0}      Return type {1}", "     ", ((PropertyInfo)m).GetGetMethod().ReturnType);
					}
					if(((PropertyInfo)m).GetSetMethod() != null)
					{
						Console.WriteLine ("{0} Set: {1}", "     ", ((PropertyInfo)m).GetSetMethod());
						Console.WriteLine ("{0}      Return type {1}", "     ", ((PropertyInfo)m).GetSetMethod().ReturnType);
					}
				}
				else
				{
					Console.WriteLine ("{0}{1}", "     Type: ", m);
				}
			}
			Console.WriteLine();
		}
	}
}
