//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Soap;

namespace MultiLevel
{
	[Serializable]
	public class Person
	{
		public string name;
		public double age;
	}

	[Serializable]
	public class Marriage
	{
		public Person husband;
		public Person wife;
	}

	[Serializable]
	public class Family
	{
		public Marriage couple;
		public Person [] children;
	}

	/// <summary>
	///    Summary description for MultiLevelSerialization.
	/// </summary>
	public class MultiLevelSerialization
	{
		public static int Main(string[] args)
		{
			Person me = new Person();
			me.age = 43;
			me.name = "Kevin Burton";

			Person wife = new Person();
			wife.age = 50;
			wife.name = "Becky Burton";

			Marriage marriage = new Marriage();
			marriage.husband = me;
			marriage.wife = wife;

			Family family = new Family();
			family.couple = marriage;

			family.children = new Person[4];
			family.children[0] = new Person();
			family.children[0].age = 20;
			family.children[0].name = "Sarah";
			family.children[1] = new Person();
			family.children[1].age = 18;
			family.children[1].name = "Ann Marie";
			family.children[2] = new Person();
			family.children[2].age = 17;
			family.children[2].name = "Mike";
			family.children[2] = new Person();
			family.children[2].age = 13;
			family.children[2].name = "Rose";

			Stream stream = File.Create("out.xml");
			SoapFormatter soap = new SoapFormatter();
			soap.Serialize(stream, family);
			stream.Close();

			stream = File.OpenRead("out.xml");
			soap = new SoapFormatter();
			family = (Family)soap.Deserialize(stream);
			stream.Close();

			return 0;
		}
	}
}
