//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

namespace MultiLevelXML
{
	public class Person
	{
		[XmlAttribute]
		public string name;
		[XmlAttribute]
		public double age;
	}

	public class Marriage
	{
		public Person husband;
		public Person wife;
	}

	public class Family
	{
		public Marriage couple;
		public Person [] children;
	}

	/// <summary>
	///    Summary description for MultiLevelSerialization.
	/// </summary>
	public class MultiLevelSerialization
	{
		public static int Main(string[] args)
		{
			Person me = new Person();
			me.age = 43;
			me.name = "Kevin";

			Person wife = new Person();
			wife.age = 50;
			wife.name = "Becky";

			Marriage marriage = new Marriage();
			marriage.husband = me;
			marriage.wife = wife;

			// Create an instance of the XmlSerializer class;
			// specify the type of object to serialize.
			XmlSerializer serializer = 
				new XmlSerializer(typeof(Marriage));
			StreamWriter writer = new StreamWriter("out1.xml");
			serializer.Serialize(writer, marriage);
			writer.Close();

			Family burton = new Family();
			burton.couple = marriage;

			burton.children = new Person[4];
			burton.children[0] = new Person();
			burton.children[0].age = 20;
			burton.children[0].name = "Sarah";
			burton.children[1] = new Person();
			burton.children[1].age = 18;
			burton.children[1].name = "Ann Marie";
			burton.children[2] = new Person();
			burton.children[2].age = 17;
			burton.children[2].name = "Mike";
			burton.children[2] = new Person();
			burton.children[2].age = 13;
			burton.children[2].name = "Rose";

			// Create an instance of the XmlSerializer class;
			// specify the type of object to serialize.
			serializer = 
				new XmlSerializer(typeof(Family));
			writer = new StreamWriter("out2.xml");
			serializer.Serialize(writer, burton);
			writer.Close();

			return 0;
		}
	}
}

