//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.IO;
 
namespace VehicleSerialization
{
	public class VehicleBuilder
	{
		public Vehicle Vehicle(string licenseNumber) 
		{
			if (licenseNumber == "0") 
			{
				Vehicle v = new Car();
				v.licenseNumber = licenseNumber;
				v.make = DateTime.Now;
				return v;
			}
			else if (licenseNumber == "1") 
			{
				Vehicle v = new Bike();
				v.licenseNumber = licenseNumber;
				v.make = DateTime.Now;
				return v;
			}
			else 
			{
				return null;
			}
		}
	}

	// The following line is not there then an exception is generated
	// Unhandled Exception: System.Exception: There was an error generating the XML document. --->
	// System.Exception: The type VehicleSerialization.Car was not expected.
	// Use the XmlInclude or SoapInclude attribute to specify types that are not known
	// statically.
	[XmlInclude(typeof(Car)), XmlInclude(typeof(Bike))]
	public abstract class Vehicle 
	{
		public string licenseNumber;
		public DateTime make;
	}
 
	public class Car : Vehicle 
	{
	}
 
	public class Bike : Vehicle 
	{
	}

	public class VehicleSerializationTest
	{
		public static int Main(string[] args)
		{
			VehicleBuilder vb = new VehicleBuilder();
			Vehicle [] va = new Vehicle[2];
			va[0] = vb.Vehicle("0");
			va[1] = vb.Vehicle("1");
			// Create an instance of the XmlSerializer class;
			// specify the type of object to serialize.
			XmlSerializer serializer = 
				new XmlSerializer(typeof(Vehicle[]));
			StreamWriter streamWriter = new StreamWriter("out.xml");
			serializer.Serialize(streamWriter, va);
			streamWriter.Close();

			StreamReader streamReader = new StreamReader("out.xml");
			va = (Vehicle [])serializer.Deserialize(streamReader);
			streamReader.Close();

			Console.WriteLine("The first item is: {0}", va[0]);
			Console.WriteLine("The second item is: {0}", va[1]);

			return 0;
		}
	}
}